# -*- coding: utf-8 -*-
#COPYRIGHT 2017 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The Business Analyst toolbox contains a set of tools that perform GIS
operations fundamental to market analysis. With the tools in this
toolbox, you can perform demographic reporting, review customer and
competitor distributions, identify key trade areas, and visualize data
variables."""
__all__ = ['ColorCodedLayer', 'CustomerDerivedTA', 'EnrichLayer',
           'SummaryReports']
__alias__ = 'ba'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Analysis toolset
@gptooldoc('ColorCodedLayer_ba', None)
def ColorCodedLayer(classification_variable=None, out_layer_name=None, classification_method=None, number_of_classes=None):
    """ColorCodedLayer_ba(classification_variable, out_layer_name, classification_method, number_of_classes)

        Creates a multi-geography level, scale dependent choropleth layer
        from a variable describing a business, demographic, consumer, or
        landscape characteristic.

     INPUTS:
      classification_variable (String):
          Select one variable to display as a color coded map. You must be
          signed in to ArcGIS Online or have Business Analyst Data installed.
      out_layer_name (String):
          The name of the color coded layer that will be added to the map.
      classification_method (String):
          The method that will be used to calculate the class breaks.

          * NATURAL_BREAKS-Natural breaks classes are based on natural
          groupings inherent in the data. Class breaks are identified that best
          group similar values and that maximize the differences between
          classes. This is the default.

          * QUANTILE-Each class contains an equal number of features. A
          quantile classification is well suited to linearly distributed data.

          * EQUAL_INTERVAL-Divides the range of attribute values into equal-
          sized sub-ranges. This allows you to specify the number of intervals,
          and ArcGIS Pro will automatically determine the class breaks based on
          the value range.

          * GEOMETRIC_INTERVAL-Creates class breaks based on class intervals
          that have a geometric series. The geometric coefficient in this
          classifier can change once (to its inverse) to optimize the class
          ranges.
      number_of_classes (String):
          The number of data classification breaks that appear on the map. The
          default value is 5."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ColorCodedLayer_ba(*gp_fixargs((classification_variable, out_layer_name, classification_method, number_of_classes), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('EnrichLayer_ba', None)
def EnrichLayer(in_features=None, out_feature_class=None, country=None, data_collection=None, variables=None, buffer_type=None, distance=None, unit=None):
    """EnrichLayer_ba(in_features, out_feature_class, country, data_collection, {variables;variables...}, {buffer_type}, {distance}, {unit})

        Enriches your data by adding demographic and landscape facts about
        the people and places that surround or are inside your data locations.
        The output is a duplicate of your input with additional attribute
        fields.The demographic and landscape information available with this
        tool can
        come from ArcGIS Online or locally installed Business Analyst data.
        Enrich Layer will consume credits if ArcGIS Online is set as the
        Business Analyst Data Source.This tool is similar to the Enrich Layer
        (Analysis) tool. However, the
        Enrich Layer (Business Analyst) tool provides an enhanced data
        variable browsing experience, and supports locally installed Business
        Analyst data (does not consume credits) in addition to ArcGIS Online
        data (consumes credits). Enrich Layer uses detailed aggregation and
        apportionment settings to
        summarize data. For more information, see Data Apportionment.

     INPUTS:
      in_features (Feature Layer):
          The features to be enriched.
      country (String):
          Select the country to specify which data variables will be
          summarized.
      data_collection (String):
          The collection of data used to enrich the input. You can specify a
          data collection without selecting any variables to enrich your data
          with all variables included in that collection. The Global country
          code only includes one data collection, KeyGlobalFacts.You can use a
          value such as Health & Personal Care
          (HealthPersonalCare), or you can use the collection name
          HealthPersonalCare.
      variables {String}:
          Select one or more variables to be summarized and added to your
          Output Feature Class. You must be signed in to ArcGIS Online or have
          Business Analyst Data installed.
      buffer_type {String}:
          Input point features must have an associated boundary polygon to
          enrich. When connected to ArcGIS Online, travel mode options are
          dynamically populated. Input line features can only use Straight Line
          distance. The default value is Straight Line.
      distance {Double}:
          The value that determines the distance or size of an area to enrich
          (i.e. a 1-mile buffer or 5-minute Walk Time). Units correspond to the
          polygon type. The default value is 1.
      unit {String}:
          The units associated with the Distance or time parameter.

     OUTPUTS:
      out_feature_class (Feature Class):
          A new layer containing both the input attributes and user selected
          attributes. User selected attributes are summarized from underlying
          demographic boundaries. Only the area inside the input boundary is
          considered."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EnrichLayer_ba(*gp_fixargs((in_features, out_feature_class, country, data_collection, variables, buffer_type, distance, unit), True)))
        return retval
    except Exception as e:
        raise e


# Reports toolset
@gptooldoc('SummaryReports_ba', None)
def SummaryReports(in_features=None, report_templates=None, reports_folder=None, summarization_options=None, single_report=None, formats=None, store_id_field=None, store_name_field=None, store_address_field=None, store_latitude_field=None, store_longitude_field=None, ring_id_field=None, area_description_field=None, title=None, subtitle=None):
    """SummaryReports_ba(in_features, report_templates;report_templates..., reports_folder, {summarization_options}, {single_report}, {formats;formats...}, {store_id_field}, {store_name_field}, {store_address_field}, {store_latitude_field}, {store_longitude_field}, {ring_id_field}, {area_description_field}, {title}, {subtitle})

        Populates and creates demographic style summary reports for any
        boundary layer, using Esri report templates.

     INPUTS:
      in_features (Feature Layer):
          The boundary layer containing one or more polygons used to create
          reports.
      report_templates (String):
          Select one or more summary report templates. You must be signed in to
          ArcGIS Online or have Business Analyst Data installed.
      reports_folder (Folder):
          Select the output location to save summary reports.
      summarization_options {String}:
          How the data will be displayed on a report.

          * INDIVIDUAL_FEATURES-Returns selected report template for each
          individual trade area polygon. This is the default.

          * WHOLE_LAYER-Returns selected report templates representing only the
          full extent of the trade area.

          * BOTH_FEATURES_AND_LAYER-Returns selected report template for both
          individual features and the whole layer.
      single_report {Boolean}:
          Generate reports per boundary or stitching all reports into a single
          report file.

          * CREATE_SINGLE_REPORT-Stitches all reports together into a single
          output.

          * CREATE_REPORT_PER_TEMPLATE-Generates separate files for each
          selected report. This is the default.
      formats {String}:
          Select the desired report output format. The default value is PDF.
      store_id_field {Field}:
          The store ID associated with each boundary.
      store_name_field {Field}:
          The store name associated with each boundary.
      store_address_field {Field}:
          The store address associated with each trade area.
      store_latitude_field {Field}:
          The field containing your latitude coordinates (y field).
      store_longitude_field {Field}:
          The field containing your longitude coordinates (x field).
      ring_id_field {Field}:
          The ring ID associated with each boundary.
      area_description_field {Field}:
          Additional descriptive information associated with each boundary.
      title {String}:
          Title on the report header.
      subtitle {String}:
          Subtitle on the report header. The default value is Prepared by
          Business Analyst Pro."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SummaryReports_ba(*gp_fixargs((in_features, report_templates, reports_folder, summarization_options, single_report, formats, store_id_field, store_name_field, store_address_field, store_latitude_field, store_longitude_field, ring_id_field, area_description_field, title, subtitle), True)))
        return retval
    except Exception as e:
        raise e


# Trade Areas toolset
@gptooldoc('CustomerDerivedTA_ba', None)
def CustomerDerivedTA(in_stores_layer=None, store_id_field=None, in_customers_layer=None, link_field=None, out_feature_class=None, method=None, rings=None, customer_aggregation_type=None, customer_weight_field=None, exclude_outlying_customers=None, cutoff_distance=None, dissolve_option=None, use_customer_centroids=None):
    """CustomerDerivedTA_ba(in_stores_layer, store_id_field, in_customers_layer, link_field, out_feature_class, method, rings;rings..., customer_aggregation_type, {customer_weight_field}, {exclude_outlying_customers}, {cutoff_distance}, {dissolve_option}, {use_customer_centroids})

        Creates trade areas around stores based on the number of customers or
        volume attribute of each customer.

     INPUTS:
      in_stores_layer (Feature Layer):
          Input point layer representing store or facility locations.
      store_id_field (Field):
          Unique ID representing a store or facility location.
      in_customers_layer (Feature Layer):
          Input point layer representing customers or patrons.
      link_field (Field):
          An ID used to assign individual customers to stores.
      method (String):
          The type of customer-derived trade area to be generated.

          * SIMPLE-Creates a generalized trade area based on the percentages of
          customers corresponding to each store.

          * AMOEBA-Connects points representing the boundary of the polygon
          trade area using natural curvature.

          * DETAILED-Connects points representing the boundary of the polygon
          trade area using straight lines.

          * DETAILED_WITH_SMOOTHING-Connects points representing the boundary
          of the polygon trade area with smoothed curves using cubic splines.
          This approach takes into account the shape and pattern of the customer
          distributions. This is the default.
      rings (Double):
          One or more values representing the percentage of customers (i.e.
          total count) or a customer attribute (i.e. total sales) assigned to
          each store. Each value represents one trade area polygon.
      customer_aggregation_type (String):
          The type of aggregation.

          * COUNT-Calculates percentage-based trade areas using the geographic
          locations of customers. This is the default.

          * WEIGHT-Calculates percentage-based trade areas using a customer
          attribute (i.e. Sales).
      customer_weight_field {Field}:
          The field used to calculate the trade areas. This is based on either
          the number of customers (count) or the calculated weighted value
          assigned to each customer.
      exclude_outlying_customers {Boolean}:
          Provides an option to exclude outlying customers from the trade area
          creation process.

          * EXCLUDE_OUTLIERS-Excludes outlying customers.

          * ALL_POINTS-Considers all customers. This is the default.
      cutoff_distance {Linear Unit}:
          The distance beyond which customers will be considered outliers, and
          therefore excluded from consideration during trade area creation.
      dissolve_option {String}:
          Creates polygons of the entire area or splits the polygons into
          individual features.

          * OVERLAP-Creates output polygons where each feature begins at zero
          and grows to satisfy the desired percentage of customers. For example,
          if you request a trade area of 50% and 70% of your customers, one
          polygon will be generated to include 0% - 50% and a second polygon
          will include all 0%-70% of customers. This is the default.

          * SPLIT-Creates output polygons for individual features based on the
          requested percentage breaks. For example, if you request a trade area
          of 50% and 70% of your customers, one polygon will be generated to
          include 0% - 50% and a second polygon will include 50%-70% of
          customers.
      use_customer_centroids {Boolean}:
          Locates the center of your customer area and calculates trade areas
          outward from this point.

          * USE_CENTROIDS-Uses centroid of customer points to calculate trade
          areas.

          * USE_STORES-Uses store location as starting point to calculate trade
          areas. This is the default.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output trade area feature class."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CustomerDerivedTA_ba(*gp_fixargs((in_stores_layer, store_id_field, in_customers_layer, link_field, out_feature_class, method, rings, customer_aggregation_type, customer_weight_field, exclude_outlying_customers, cutoff_distance, dissolve_option, use_customer_centroids), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject